/*
 * avc.cc - An AV/C target class
 *
 * Copyright (C) 2002 Dan Dennedy <dan@dennedy.org>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <sys/types.h>
#include <sys/time.h>

#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>

#include <libraw1394/raw1394.h>
#include <libavc1394/avc1394.h>

#include "avc.h"

IEC61883Stream *AvcTarget::m_ts(NULL);


unsigned char g_signal_mode = 0x01; // 0x01;	// 0x1 =DVHS, 0x5 = SD 525-60, TODO: get from media
unsigned char g_transport_mode = AVC1394_VCR_CMD_WIND;
unsigned char g_transport_state = AVC1394_VCR_OPERAND_WIND_STOP;
unsigned char g_power = 0x70;		// powered on
byte_t connect_block[5];		// AVC_CONNECT state

void AvcTarget::play(avc1394_cmd_rsp * cr, float rate)
{
    cr->status = AVC1394_RESP_ACCEPTED;
    printf("PLAY %f\n", rate);
    if (g_transport_mode == AVC1394_VCR_CMD_PLAY)
	m_ts->play(rate);
    // TODO: get channel from oPCR
    // TODO: get isPAL from file
    else if (m_ts->init(32, false))
	m_ts->play(rate);
    g_transport_mode = AVC1394_VCR_CMD_PLAY;
    g_transport_state = cr->operand[0];
}

/**** subunit handlers ****/
int AvcTarget::subunit_control(avc1394_cmd_rsp *cr)
{
    switch (cr->opcode) {
    case AVC1394_VCR_CMD_PLAY:
	switch (cr->operand[0]) {
	case AVC1394_VCR_OPERAND_PLAY_SLOWEST_FORWARD:
	    play(cr, 0.3);
	    break;
	case AVC1394_VCR_OPERAND_PLAY_SLOW_FORWARD_6:
	    play(cr, 0.4);
	    break;
	case AVC1394_VCR_OPERAND_PLAY_SLOW_FORWARD_5:
	    play(cr, 0.5);
	    break;
	case AVC1394_VCR_OPERAND_PLAY_SLOW_FORWARD_4:
	    play(cr, 0.6);
	    break;
	case AVC1394_VCR_OPERAND_PLAY_SLOW_FORWARD_3:
	    play(cr, 0.7);
	    break;
	case AVC1394_VCR_OPERAND_PLAY_SLOW_FORWARD_2:
	    play(cr, 0.8);
	    break;
	case AVC1394_VCR_OPERAND_PLAY_SLOW_FORWARD_1:
	    play(cr, 0.9);
	    break;
	case AVC1394_VCR_OPERAND_PLAY_FORWARD:
	case AVC1394_VCR_OPERAND_PLAY_X1_FORWARD:
	    play(cr, 1.0);
	    break;
	case AVC1394_VCR_OPERAND_PLAY_FAST_FORWARD_1:
	    play(cr, 2.0);
	    break;
	case AVC1394_VCR_OPERAND_PLAY_FAST_FORWARD_2:
	    play(cr, 3.0);
	    break;
	case AVC1394_VCR_OPERAND_PLAY_FAST_FORWARD_3:
	    play(cr, 4.0);
	    break;
	case AVC1394_VCR_OPERAND_PLAY_FAST_FORWARD_4:
	    play(cr, 5.0);
	    break;
	case AVC1394_VCR_OPERAND_PLAY_FAST_FORWARD_5:
	    play(cr, 6.0);
	    break;
	case AVC1394_VCR_OPERAND_PLAY_FAST_FORWARD_6:
	    play(cr, 7.0);
	    break;
	case AVC1394_VCR_OPERAND_PLAY_FASTEST_FORWARD:
	    play(cr, 8.0);
	    break;

	case AVC1394_VCR_OPERAND_PLAY_REVERSE_PAUSE:
	case AVC1394_VCR_OPERAND_PLAY_FORWARD_PAUSE:
	    cr->status = AVC1394_RESP_ACCEPTED;
	    printf("PAUSE PLAY\n");
	    if (g_transport_mode == AVC1394_VCR_CMD_PLAY)
		m_ts->pause();
	    else if (m_ts->init(32, false)) {
		m_ts->play(1.0);
		m_ts->pause();
	    }
	    g_transport_mode = AVC1394_VCR_CMD_PLAY;
	    g_transport_state = cr->operand[0];
	    break;

	case AVC1394_VCR_OPERAND_PLAY_SLOWEST_REVERSE:
	    play(cr, -0.3);
	    break;
	case AVC1394_VCR_OPERAND_PLAY_SLOW_REVERSE_6:
	    play(cr, -0.4);
	    break;
	case AVC1394_VCR_OPERAND_PLAY_SLOW_REVERSE_5:
	    play(cr, -0.5);
	    break;
	case AVC1394_VCR_OPERAND_PLAY_SLOW_REVERSE_4:
	    play(cr, -0.6);
	    break;
	case AVC1394_VCR_OPERAND_PLAY_SLOW_REVERSE_3:
	    play(cr, -0.7);
	    break;
	case AVC1394_VCR_OPERAND_PLAY_SLOW_REVERSE_2:
	    play(cr, -0.8);
	    break;
	case AVC1394_VCR_OPERAND_PLAY_SLOW_REVERSE_1:
	    play(cr, -0.9);
	    break;
	case AVC1394_VCR_OPERAND_PLAY_REVERSE:
	case AVC1394_VCR_OPERAND_PLAY_X1_REVERSE:
	    play(cr, -1.0);
	    break;
	case AVC1394_VCR_OPERAND_PLAY_FAST_REVERSE_1:
	    play(cr, -2.0);
	    break;
	case AVC1394_VCR_OPERAND_PLAY_FAST_REVERSE_2:
	    play(cr, -3.0);
	    break;
	case AVC1394_VCR_OPERAND_PLAY_FAST_REVERSE_3:
	    play(cr, -4.0);
	    break;
	case AVC1394_VCR_OPERAND_PLAY_FAST_REVERSE_4:
	    play(cr, -5.0);
	    break;
	case AVC1394_VCR_OPERAND_PLAY_FAST_REVERSE_5:
	    play(cr, -6.0);
	    break;
	case AVC1394_VCR_OPERAND_PLAY_FAST_REVERSE_6:
	    play(cr, -7.0);
	    break;
	case AVC1394_VCR_OPERAND_PLAY_FASTEST_REVERSE:
	    play(cr, -8.0);
	    break;

	case AVC1394_VCR_OPERAND_PLAY_NEXT_FRAME:
	    cr->status = AVC1394_RESP_ACCEPTED;
	    printf("PLAY NEXT FRAME\n");
	    g_transport_mode = AVC1394_VCR_CMD_PLAY;
	    g_transport_state = cr->operand[0];
	    break;

	case AVC1394_VCR_OPERAND_PLAY_PREVIOUS_FRAME:
	    cr->status = AVC1394_RESP_ACCEPTED;
	    printf("PLAY PREVIOUS FRAME\n");
	    g_transport_mode = AVC1394_VCR_CMD_PLAY;
	    g_transport_state = cr->operand[0];
	    break;

	default:
	    fprintf(stderr, "play mode 0x%02x non supported\n",
		    cr->operand[0]);
	    return 0;
	}
	break;

    case AVC1394_VCR_CMD_RECORD:
	switch (cr->operand[0]) {
	case AVC1394_VCR_OPERAND_RECORD_RECORD:
	    cr->status = AVC1394_RESP_ACCEPTED;
	    printf("AVC1394_VCR_OPERAND_RECORD_RECORD\n");
	    // TODO: get channel from iPCR
	    // TODO: get isPAL from g_signal_mode
	    
	    if (g_transport_mode == AVC1394_VCR_CMD_RECORD) {
		printf("Already in recording mode - continue record\n");
		m_ts->record();
	    } else if (m_ts->init(32, false)) {
		printf("Started brand new recording stream\n");
		m_ts->record();
	    }
	    g_transport_mode = AVC1394_VCR_CMD_RECORD;
	    g_transport_state = cr->operand[0];
	    break;

	case AVC1394_VCR_OPERAND_RECORD_PAUSE:
	    cr->status = AVC1394_RESP_ACCEPTED;
	    printf("PAUSE RECORD\n");
	    m_ts->pause();
	    g_transport_mode = AVC1394_VCR_CMD_RECORD;
	    g_transport_state = cr->operand[0];
	    break;

	default:
	    fprintf(stderr, "record mode 0x%02x non supported\n",
		    cr->operand[0]);
	    return 0;
	}
	break;

    case AVC1394_VCR_CMD_WIND:
	switch (cr->operand[0]) {
	case AVC1394_VCR_OPERAND_WIND_STOP:
	    cr->status = AVC1394_RESP_ACCEPTED;
	    printf("STOP\n");
	    m_ts->stop();
	    g_transport_mode = AVC1394_VCR_CMD_WIND;
	    g_transport_state = cr->operand[0];
	    break;

	case AVC1394_VCR_OPERAND_WIND_REWIND:
	case AVC1394_VCR_OPERAND_WIND_HIGH_SPEED_REWIND:
	    cr->status = AVC1394_RESP_ACCEPTED;
	    printf("REWIND\n");
	    m_ts->rewind();
	    g_transport_mode = AVC1394_VCR_CMD_WIND;
	    g_transport_state = AVC1394_VCR_OPERAND_WIND_REWIND;
	    break;

	case AVC1394_VCR_OPERAND_WIND_FAST_FORWARD:
	    cr->status = AVC1394_RESP_ACCEPTED;
	    printf("FAST FORWARD\n");
	    m_ts->ff();
	    g_transport_mode = AVC1394_VCR_CMD_WIND;
	    g_transport_state = cr->operand[0];
	    break;

	default:
	    fprintf(stderr, "wind mode 0x%02x non supported\n",
		    cr->operand[0]);
	    return 0;
	}
	break;

    case AVC1394_VCR_CMD_FORWARD:
	switch (cr->operand[0]) {
	case AVC1394_VCR_MEASUREMENT_INDEX:
	    {
		FILE *f = m_ts->getFileList().nextFile();
		if (f == NULL)
		    m_ts->stop();
		break;
	    }
	default:
	    fprintf(stderr, "forward measurement 0x%02x non supported\n",
		    cr->operand[0]);
	    return 0;
	}
	break;

    case AVC1394_VCR_CMD_BACKWARD:
	switch (cr->operand[0]) {
	case AVC1394_VCR_MEASUREMENT_INDEX:
	    {
		FILE *f = m_ts->getFileList().prevFile();
		if (f == NULL)
		    m_ts->stop();
		break;
	    }
	default:
	    fprintf(stderr, "backward measurement 0x%02x non supported\n",
		    cr->operand[0]);
	    return 0;
	}
	break;

    case AVC1394_VCR_CMD_TAPE_RECORDING_FORMAT:
	cr->status = AVC1394_RESP_ACCEPTED;
	cr->operand[0] = 1 << 3;
	cr->operand[1] = 1 << 5;
	cr->operand[2] = 0x0;
	cr->operand[3] = 0x0;
	cr->operand[4] = 0x1;
	cr->operand[5] = 0x0;
	cr->operand[6] = 0x0;
	cr->operand[7] = 0x0;
	cr->operand[8] = 0x0;
	break;

    case AVC1394_VCR_CMD_RECORDING_SPEED:
	cr->status = AVC1394_RESP_ACCEPTED;
	cr->operand[0] = 0x6f;
	break;

    default:
	fprintf(stderr, "subunit control command 0x%02x non supported\n",
		cr->opcode);
	return 0;
    }
    return 1;
}


int AvcTarget::subunit_status(avc1394_cmd_rsp * cr)
{
    switch (cr->opcode) {
    case AVC1394_VCR_CMD_INPUT_SIGNAL_MODE:
    case AVC1394_VCR_CMD_OUTPUT_SIGNAL_MODE:
	cr->status = AVC1394_RESP_STABLE;
	cr->operand[0] = g_signal_mode;
	break;

    case AVC1394_VCR_CMD_TRANSPORT_STATE:
	cr->status = AVC1394_RESP_STABLE;

	/* check consistency against underlying stream engine */
	if (g_transport_mode == AVC1394_VCR_CMD_PLAY && !m_ts->isPlaying()) {
	    g_transport_mode = AVC1394_VCR_CMD_WIND;
	    g_transport_state = AVC1394_VCR_OPERAND_WIND_STOP;
	} else if (g_transport_mode == AVC1394_VCR_CMD_WIND
		   && !m_ts->isPlaying()) {
	    g_transport_mode = AVC1394_VCR_CMD_WIND;
	    g_transport_state = AVC1394_VCR_OPERAND_WIND_STOP;
	} else if (g_transport_mode == AVC1394_VCR_CMD_RECORD &&
		!m_ts->isRecording()) {
	    g_transport_mode = AVC1394_VCR_CMD_WIND;
	    g_transport_state = AVC1394_VCR_OPERAND_WIND_STOP;
	} else {
	    printf("Unknown combination in TRANSPORT_STATE\n");
	    g_transport_mode = AVC1394_VCR_CMD_WIND;
	    g_transport_state = AVC1394_VCR_OPERAND_WIND_STOP;
	}

	cr->opcode = g_transport_mode;
	cr->operand[0] = g_transport_state;
	break;

    case AVC1394_VCR_CMD_RELATIVE_TIME_COUNTER:
    case AVC1394_VCR_CMD_TIME_CODE:
	cr->status = AVC1394_RESP_STABLE;
	cr->operand[0] = AVC1394_VCR_OPERAND_RECORDING_TIME_STATUS;
	// TODO: extract timecode from media or use time elapsed since start of app
	cr->operand[1] = 1;	//frames
	cr->operand[2] = 2;	//seconds
	cr->operand[3] = 3;	//minutes
	cr->operand[4] = 4;	//hours
	break;

    case AVC1394_VCR_CMD_MEDIUM_INFO:
	cr->status = AVC1394_RESP_STABLE;
	cr->operand[0] = AVC1394_VCR_OPERAND_MEDIUM_INFO_VHS;
	cr->operand[1] = 0x50; // add later AVC1394_VCR_OPERAND_MEDIUM_INFO_OK;
	break;

    case AVC1394_VCR_CMD_TAPE_PLAYBACK_FORMAT:
	cr->status = AVC1394_RESP_STABLE;
	cr->operand[0] = 1 << 3;
	cr->operand[1] = 1 << 5;
	cr->operand[2] = 0x0;
	cr->operand[3] = 0x0;
	cr->operand[4] = 0x1;
	cr->operand[5] = 0x0;
	cr->operand[6] = 0x0;
	cr->operand[7] = 0x0;
	cr->operand[8] = 0x0;
	break;

    default:
	fprintf(stderr, "subunit status command 0x%02x not supported\n",
		cr->opcode);
	return 0;
    }
    return 1;
}


int AvcTarget::subunit_inquiry(avc1394_cmd_rsp * cr)
{
    printf("AvcTarget::subunit_inquiry\n");

    switch (cr->opcode) {
    case AVC1394_VCR_CMD_PLAY:
    case AVC1394_VCR_CMD_RECORD:
    case AVC1394_VCR_CMD_WIND:
    case AVC1394_VCR_CMD_FORWARD:
    case AVC1394_VCR_CMD_BACKWARD:
    case AVC1394_VCR_CMD_OUTPUT_SIGNAL_MODE:
    case AVC1394_VCR_CMD_INPUT_SIGNAL_MODE:
    case AVC1394_VCR_CMD_TRANSPORT_STATE:
    case AVC1394_VCR_CMD_TIME_CODE:
    case AVC1394_VCR_CMD_MEDIUM_INFO:
    case AVC1394_VCR_CMD_RELATIVE_TIME_COUNTER:
	cr->status = AVC1394_RESP_IMPLEMENTED;
	break;
    default:
	fprintf(stderr, "subunit inquiry command 0x%02x not supported\n",
		cr->opcode);
	return 0;
    }
    return 1;
}


/**** Unit handlers ****/
int AvcTarget::unit_control(avc1394_cmd_rsp * cr)
{
    switch (cr->opcode) {
	case AVC1394_CMD_POWER:
	    cr->opcode = AVC1394_RESP_ACCEPTED;
	    g_power = cr->operand[0];
	    if (g_power == 0x60)
		m_ts->stop();
	    break;

	case AVC1394_CMD_CONNECT:
	    cr->opcode = AVC1394_RESP_ACCEPTED;
	    printf("AVC1394_CMD_CONNECT: %02x%02x:%02d -> %02x%02x:%02d\n",
		    cr->operand[1] >> 3, cr->operand[1] & 0x7,
		    cr->operand[2],
		    cr->operand[3] >> 3, cr->operand[3] & 0x7,
		    cr->operand[4]);

	    // remember the connections
	    connect_block[0] = cr->operand[1]; // source subunit
	    connect_block[1] = cr->operand[2]; // source plug
	    connect_block[2] = cr->operand[3]; // dest subunit
	    connect_block[3] = cr->operand[4]; // dest plug
	    break;

	default:
	    fprintf(stderr, "unit control command 0x%02x not supported\n",
		    cr->opcode);
	    return 0;
    }
    return 1;
}


int AvcTarget::unit_status(avc1394_cmd_rsp * cr)
{

    switch (cr->opcode) {
	case AVC1394_CMD_VENDOR_DEPENDENT:
	    // vendor extension
	    printf("VENDOR: %02x : %02x %02x %02x %02x %02x %02x %02x %02x %02x\n",
		    cr->status,
		    cr->operand[0], cr->operand[1],
		    cr->operand[2], cr->operand[3],
		    cr->operand[4], cr->operand[5],
		    cr->operand[6], cr->operand[7],
		    cr->operand[8]);
	    cr->status = AVC1394_RESP_STABLE; // NOT_IMPLEMENTED;
	    break;

	case AVC1394_CMD_UNIT_INFO:
	    cr->status = AVC1394_RESP_STABLE;
	    cr->operand[0] = AVC1394_OPERAND_UNIT_INFO_EXTENSION_CODE;
	    cr->operand[1] = AVC1394_SUBUNIT_TAPE_RECORDER;
	    cr->operand[2] = 0xff;
	    break;

	case AVC1394_CMD_SUBUNIT_INFO:
	    {
		int page = (cr->operand[0] >> 4) & 7;
		if (page == 0) {
		    cr->status = AVC1394_RESP_STABLE;
		    cr->operand[0] =
			(page << 4) | AVC1394_OPERAND_UNIT_INFO_EXTENSION_CODE;
		    cr->operand[1] = AVC1394_SUBUNIT_TAPE_RECORDER << 3;
		    cr->operand[2] = 0xff;
		} else {
		    fprintf(stderr, "invalid page %d for subunit\n", page);
		    return 0;
		}
		break;
	    }

	case AVC1394_CMD_OUTPUT_PLUG_SIGNAL_FORMAT:
	    printf("AVC1394_CMD_OUTPUT_PLUG_SIGNAL_FORMAT *************\n");
	    cr->status = AVC1394_RESP_STABLE;
	    cr->operand[1] = (0x2 << 6) | (0x20);	// fmt
	    cr->operand[2] = 0x0; // 80;	//time shifted flag
	    cr->operand[3] = 0;	//reserved
	    cr->operand[4] = 0;	//reserved
	    break;

	case AVC1394_CMD_POWER:
	    cr->status = AVC1394_RESP_STABLE;
	    cr->operand[0] = g_power;
	    break;

	case AVC1394_CMD_CONNECT:
	    printf("AVC1394_CMD_CONNECT_STATUS\n");
	    cr->status = AVC1394_RESP_STABLE;
	    printf("%02x %02x %02x %02x %02x\n",
		    cr->operand[0],
		    cr->operand[1],
		    cr->operand[2],
		    cr->operand[3],
		    cr->operand[4]);

	    if (cr->operand[4] == 0xFE) {
		printf("Requesting info for SOURCE plug\n");
	    } else {
		printf("Requesting info for DESTINATION plug\n");
		cr->operand[0] = 0x3F << 2;
		cr->operand[1] = connect_block[0];
		cr->operand[2] = connect_block[1];
		cr->operand[3] = connect_block[2];
		cr->operand[4] = connect_block[3];
	    }
	    break;

	default:
    cr->operand[1] = 0xff;
    cr->operand[2] = 0xff;
    cr->operand[3] = 0xff;
    cr->operand[4] = 0xff;
	    fprintf(stderr, "unit status command 0x%02x not supported\n",
		    cr->opcode);
	    return 0;
    }
    return 1;
}


int AvcTarget::unit_inquiry(avc1394_cmd_rsp * cr)
{
    switch (cr->opcode) {
    case AVC1394_CMD_SUBUNIT_INFO:
    case AVC1394_CMD_UNIT_INFO:
    case AVC1394_CMD_OUTPUT_PLUG_SIGNAL_FORMAT:
	cr->status = AVC1394_RESP_IMPLEMENTED;
	break;
    default:
	fprintf(stderr, "unit inquiry command 0x%02x not supported\n",
		cr->opcode);
	return 0;
    }
    return 1;
}


/**** primary avc1394 target callback ****/
int AvcTarget::command_handler(struct avc1394_command_response *cr)
{
#ifndef DEBUG
    printf("command_handler: %02x : %02x (%02x/%02x) : %02x\n",
	    cr->opcode, cr->status,
	    cr->subunit_type, cr->subunit_id,
	    cr->operand[0]);
#endif

    switch (cr->subunit_type) {
    case AVC1394_SUBUNIT_TAPE_RECORDER:
	if (cr->subunit_id != 0) {
	    fprintf(stderr, "subunit id 0x%02x not supported\n",
		    cr->subunit_id);
	    return 0;
	}

	switch (cr->status) {

	case AVC1394_CTYP_CONTROL:
	    return subunit_control(cr);
	    break;
	case AVC1394_CTYP_STATUS:
	    return subunit_status(cr);
	    break;
	case AVC1394_CTYP_GENERAL_INQUIRY:
	    return subunit_inquiry(cr);
	    break;
	}
	break;

    case AVC1394_SUBUNIT_UNIT:
	switch (cr->status) {
	case AVC1394_CTYP_CONTROL:
	    return unit_control(cr);
	    break;
	case AVC1394_CTYP_STATUS:
	    return unit_status(cr);
	    break;
	case AVC1394_CTYP_GENERAL_INQUIRY:
	    return unit_inquiry(cr);
	    break;
	}
	break;

    default:
	fprintf(stderr, "subunit type 0x%02x not supported\n",
		cr->subunit_type);
	return 0;
    }
    return 1;
}

AvcTarget::AvcTarget(raw1394handle_t handle, IEC61883Stream *ts):
m_handle(handle)
{
    m_ts = ts;
    avc1394_init_target(m_handle, command_handler);
    memset(&connect_block, 0x00, sizeof(connect_block));
}

AvcTarget::~AvcTarget(void)
{
    avc1394_close_target(m_handle);
}
