/*
 *  filelist.h - Multifile I/O class
 *
 *  Copyright (C) 2002 Dan Dennedy <dan@dennedy.org>
 *   
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef FILELIST_H
#define FILELIST_H

#include <string>
using std::string;
#include <vector>
using std::vector;

#include <stdio.h>

class FileList {
  private:
    string m_dirName;
    vector < string > m_list;
    int m_pos;
    FILE *m_file;
    off_t m_fileLength;

  public:
    FileList(const char *dirName);
    void rescan(void);
    FILE *currentFile(void) {
	return m_file;
    };
    FILE *nextFile(void);
    FILE *prevFile(void);
    bool seek(size_t length, float rate);
    bool read(unsigned char *buffer, size_t length);
    bool write(unsigned char *buffer, size_t length);
    bool isAtBegin(void);
    bool isAtEnd(void);
    bool newFile(const string extension);
    void commit(void);
    void reset(void);
    void ff(void);
};

#endif
