/*
 *  iec61883.cc - IEC 61883 Stream classes
 *
 *  Copyright (C) 2002 Mass Digital LLC and Dan Dennedy <dan@dennedy.org>
 *   
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef IEC61883_H
#define IEC61883_H

#include <string>
using std::string;

#include <stdio.h>
#include <pthread.h>
#include <libraw1394/raw1394.h>

#include "filelist.h"

class IEC61883Stream {
  protected:
    pthread_mutex_t m_mutex;
    pthread_t m_thread;
    FileList & m_fileList;
    float m_rate;
    bool m_isPlaying;
    bool m_isRecording;
    bool m_cancel;

  public:
    IEC61883Stream(FileList & list);
    ~IEC61883Stream(void);

    virtual bool init(unsigned int channel, bool isPAL) = 0;
    virtual void shutdown(void) = 0;

    void play(float rate = 1.0);
    void pause(void);
    void stop(void);
    void record(void);
    bool isRecording(void) {
	return m_isRecording;
    } bool isPlaying(void) {
	return m_isPlaying;
    }
    void rewind(void) {
	stop();
	m_fileList.reset();
    }
    void ff(void) {
	stop();
	m_fileList.ff();
    };
    FileList & getFileList(void) {
	return m_fileList;
    }

  protected:
    virtual void transmit(void) = 0;
    virtual void receive(void) = 0;

  private:
    static void *transmitBootstrap(void *ptr);
    void transmitWrapper();
    static void *receiveBootstrap(void *ptr);
    void receiveWrapper();
};


class MPEG1394Stream:public IEC61883Stream {
  private:
    unsigned int m_channel;
    raw1394handle_t m_handle;
    int m_HDVR;

    static raw1394_iso_disposition
	MPEG1394Stream::transmitHandler(raw1394handle_t handle,
					unsigned char *data,
					unsigned int *len,
					unsigned char *tag,
					unsigned char *sy, int cycle,
					unsigned int dropped);
    static raw1394_iso_disposition receiveHandler(raw1394handle_t handle,
						  unsigned char *data,
						  unsigned int len,
						  unsigned char channel,
						  unsigned char tag,
						  unsigned char sy,
						  unsigned int cycle,
						  unsigned int dropped);

  public:
    MPEG1394Stream(raw1394handle_t handle, int port, FileList & list);
    ~MPEG1394Stream(void);

    bool init(unsigned int channel, bool isPAL);
    void stop(void);
    void shutdown(void);

  protected:
    void transmit(void);
    void receive(void);
    int findHDVR(void);

};

#endif
