/*
 *  plug.cc - An IEC 61883 Connection Managment Procedures class
 *
 *  Copyright (C) 2002 Dan Dennedy <dan@dennedy.org>
 *   
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <string>
#include <iostream>
using std::cerr;
using std::cout;
using std::endl;

/* standard system includes */
#include <unistd.h>
#include <string.h>
#include <stdlib.h>

/* linux1394 includes */
#include <libraw1394/raw1394.h>
#include <libavc1394/plug1394.h>

#include "plug.h"

Connection::Connection(raw1394handle_t handle):m_handle(handle)
{
    struct oMPR o_mpr;
    struct oPCR o_pcr;
    struct iMPR i_mpr;
    struct iPCR i_pcr;

    o_mpr.n_plugs = 0;
    o_mpr.data_rate = DATA_RATE_400;
    o_mpr.bcast_channel = 63;

    o_pcr.online = 0;
    o_pcr.bcast_connection = 0;
    o_pcr.n_p2p_connections = 0;
    o_pcr.data_rate = DATA_RATE_400;
    o_pcr.overhead_id = OVERHEAD_480;
    o_pcr.channel = 63;
    /* DV contains 120 quadlets for DIF block plus 2 quadlet of CIP header */
    o_pcr.payload = 146;
    plug1394_register_outputs(m_handle, o_mpr, o_pcr);

    i_mpr.n_plugs = 1;
    i_mpr.data_rate = DATA_RATE_400;

    i_pcr.online = 1;
    i_pcr.bcast_connection = 0;
    i_pcr.n_p2p_connections = 0;
    i_pcr.channel = 32;
    plug1394_register_inputs(m_handle, i_mpr, i_pcr);
}

Connection::~Connection(void)
{
    cout << "unregistering local plugs" << endl;
    plug1394_unregister_outputs(m_handle);
    plug1394_unregister_inputs(m_handle);
}


/** Set the channel number for the receiving device.
 
   \param node    The node ID of a device on the 1394 bus.
   \param channel The isochronous channel number the device should receive on.
   \return success/failure
 */
bool Connection::establishPointToPoint(nodeaddr_t node,
				       unsigned int channel)
{
    struct iMPR i_mpr;
    struct iPCR i_pcr;

    printf("Connection::establishPointToPoint\n");

    try {
	if (plug1394_get_iMPR(m_handle, node, &i_mpr) < 0)
	    throw "failed to get the iMPR plug";

	if (i_mpr.n_plugs == 0)
	    throw "the receiving device does not have any input plugs";

	/* just use the first plug */
	/* TODO: cycle thru all plugs to find first online plug */
	if (plug1394_get_iPCR0(m_handle, node, &i_pcr) < 0)
	    throw "Failed to get the iPCR[0] plug";

	/* verify the plug is online */
	/*
	   if (i_pcr.online == 0)
	   {
	   fprintf( stderr, "The receiving device is not online.\n");
	   return 0;
	   }
	 */
	/* if a connection already exists, then we must not change channel
	   and simply overlay the existing connection */
	if (i_pcr.n_p2p_connections == 0)
	    i_pcr.channel = channel;
	/* add a connection */
	i_pcr.n_p2p_connections++;

	/* finally, set the plug */
	if (plug1394_set_iPCR0(m_handle, node, i_pcr) < 0)
	    throw "failed to set the iPCR[0] plug";
    }
    catch(const char *message) {
	cerr << message << endl;
	return false;
    }

    return true;
}


/** Restore the channel number for the receiving device.
 
   \param node    The node ID of a device on the 1394 bus.
   \param channel The isochronous channel number the device should receive on.
   \return success/failure
 */
bool Connection::breakPointToPoint(nodeaddr_t node, unsigned int channel)
{
    struct iPCR i_pcr;

    try {
	if (plug1394_get_iPCR0(m_handle, node, &i_pcr) < 0)
	    throw "failed to get the iPCR[0] plug";

	if (i_pcr.n_p2p_connections == 0)
	    throw "the iPCR[0] plug does not have an existing connection";

	i_pcr.n_p2p_connections--;
	if (plug1394_set_iPCR0(m_handle, node, i_pcr) < 0)
	    throw "failed to set the iPCR[0] plug";
    }
    catch(const char *message) {
	cerr << message << endl;
	return false;
    }

    return true;
}

bool Connection::establishBroadcast(nodeaddr_t node, unsigned int channel)
{


    return false;
}

bool Connection::breakBroadcast(nodeaddr_t node, unsigned int channel)
{



    return false;
}
