/*
 *  plug.h - An IEC 61883 Connection Managment Procedures class
 *
 *  Copyright (C) 2002 Dan Dennedy <dan@dennedy.org>
 *   
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef PLUG_H
#define PLUG_H

#include <libraw1394/raw1394.h>

class Connection {
  private:
    raw1394handle_t m_handle;

  public:
	/** Create a Connection object
	
	   \param handle  A raw1394 handle.
	*/
    Connection(raw1394handle_t handle);
    ~Connection(void);

	/** Set the channel number for the receiving device.
	 
	   \param node    The node ID of a device on the 1394 bus.
	   \param channel The isochronous channel number the device should receive on.
	   \return 1 for success, 0 for error.
	*/
      bool
      establishBroadcast( nodeaddr_t node, unsigned int channel);


	/** Restore the channel number for the receiving device.
	 
	   \param node    The node ID of a device on the 1394 bus.
	   \param channel The isochronous channel number the device should receive on.
	   \return 1 for success, 0 for error.
	*/
      bool
      breakBroadcast( nodeaddr_t node, unsigned int channel);


	/** Set the channel number for the receiving device.
	 
	   \param node    The node ID of a device on the 1394 bus.
	   \param channel The isochronous channel number the device should receive on.
	   \return 1 for success, 0 for error.
	*/
     bool establishPointToPoint(nodeaddr_t node, unsigned int channel);


	/** Restore the channel number for the receiving device.
	 
	   \param node    The node ID of a device on the 1394 bus.
	   \param channel The isochronous channel number the device should receive on.
	   \return 1 for success, 0 for error.
	*/
     bool breakPointToPoint(nodeaddr_t node, unsigned int channel);
};

#endif
