/*
 * rom.h - A custom Configuration ROM class
 *
 * Copyright (C) 2002 Dan Dennedy <dan@dennedy.org>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <iostream>
using std::cout;
using std::cerr;
using std::endl;

#include <errno.h>
#include <libraw1394/raw1394.h>
#include <libavc1394/rom1394.h>
#include "rom.h"

ConfigROM::ConfigROM(raw1394handle_t handle):m_handle(handle)
{
    int retval, i;
    quadlet_t rom[0x100];
    size_t rom_size;
    rom1394_directory dir;
    char *leaf;

    /* get the current rom image */
    retval =
	raw1394_get_config_rom(m_handle, m_originalROM, 0x100, &rom_size,
			       &m_version);
    retval =
	raw1394_get_config_rom(m_handle, rom, 0x100, &rom_size,
			       &m_version);

    /* add an AV/C unit directory */
    dir.node_capabilities = 0x000083c0;
    dir.unit_spec_id = 0x0000a02d;
    dir.unit_sw_version = 0x00010001;

    leaf = "DDR1394";
    dir.nr_textual_leafs = 1;
    dir.textual_leafs = &leaf;

    // added Apple copied
    dir.model_id = 0x0000000a;
    dir.vendor_id = 0x00000a27;

    /* manipulate the rom */
    retval = rom1394_add_unit(rom, &dir);

    /* get the computed size of the rom image */
    rom_size = rom1394_get_size(rom);

    /* convert computed rom size from quadlets to bytes before update */
    rom_size *= sizeof(quadlet_t);
    retval = raw1394_update_config_rom(m_handle, rom, rom_size, m_version);

    // raw1394_reset_bus(m_handle);
}

ConfigROM::~ConfigROM(void)
{
    cout << "restoring original configROM" << endl;

    quadlet_t rom[0x100];
    size_t rom_size;

    raw1394_get_config_rom(m_handle, rom, 0x100, &rom_size, &m_version);

    rom_size = rom1394_get_size(m_originalROM);
    rom_size *= sizeof(quadlet_t);

    int result =
	raw1394_update_config_rom(m_handle, m_originalROM, rom_size,
				  m_version);
    if (result == -1)
	cerr << "failed to restore configROM " << errno << endl;
    else if (result == -2)
	cerr << "failed to restore configROM -- too big." << endl;

    //raw1394_reset_bus(m_handle);
}
